<#
.SYNOPSIS
    Domain name WhoIs

.DESCRIPTION
    Performs a domain name lookup and returns information such as
    domain availability (creation and expiration date),
    domain ownership, name servers, etc.

.PARAMETER domain
    Specifies the domain name (enter the domain name without http:// and www (e.g. power-shell.com))

.EXAMPLE
    WhoIs -domain power-shell.com
    whois power-shell.com

.NOTES
    File Name: whois.ps1
    Author: Nikolay Petkov
    Blog: http://power-shell.com
    Last Edit: 12/20/2014
    Made by: KornKolio (https://gallery.technet.microsoft.com/WHOIS-PowerShell-Function-ed69fde6)
    Add it to your $profile, line: . PATH\whois.ps1
    Or dot-source it: . PATH\whois.ps1
    You may need to Set-ExecutionPolicy
    #
    # NOT WORKING
    #
.LINK
    http://power-shell.com
#>
# Function WhoIs {
param([string]$MyArgs)
# param (
#    [
#				Parameter(
#        Mandatory = $True,
#        HelpMessage = 'Please enter domain name (e.g. microsoft.com)'
#				)
#    ]
#    [string]$Domain
# )
$Domain = "$MyArgs $args"
$FileVersion = "0.0.5"
Say "Get Whois $FileVersion"
if (!($Domain)) {
    Say "Please enter domain name (e.g. microsoft.com)"
    Say "WHOIS <domainname>"
    break
}
Say -ForegroundColor Green "Connecting to Web Services URL..."
try {
    # Retrieve the data from web service WSDL
    If ($whois = Invoke-RestMethod -Uri "http://www.webservicex.net/whois.asmx?WSDL" -Body $Domain -ContentType 'application/text') {
        #Invoke-RestMethod -Uri $hookUrl -Method Post -Body $content -ContentType 'application/json'
        #If ($whois = New-WebServiceProxy -uri "http://www.webservicex.net/whois.asmx?WSDL") {
        Write-Host "Ok" -ForegroundColor Green
    }
    else {
        Write-Host "Error" -ForegroundColor Red
    }
    Write-Host "Gathering $Domain data..." -ForegroundColor Green
    # Return the data
    (
        ($whois.getwhois("=$Domain")).Split("<<<")[0]
    )
}
catch {
    Write-Host "Please enter valid domain name (e.g. microsoft.com)." -ForegroundColor Red
}

#}
# end function WhoIs
